<?php

/**
 * @file
 * Contains \Drupal\tmgmt_xconnect\Plugin\tmgmt\Translator\XConnectTranslator.
 */

namespace Drupal\tmgmt_xconnect\Plugin\tmgmt\Translator;

use Amplexor\XConnect\Request;
use Amplexor\XConnect\Request\File\ZipFile;
use Amplexor\XConnect\Service\FtpService;
use Amplexor\XConnect\Service\SFtpService;
use Drupal\Component\Utility\Xss;
use Drupal\tmgmt\TranslatorPluginBase;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\Translator\AvailableResult;
use Drupal\tmgmt\Translator\TranslatableResult;

/**
 * Amplexor X-Connect translator plugin.
 *
 * @link http://goo.gl/VW0KK6 Translator plugin for the Euroscript Global Content Management (GCM) language services.@endlink
 * @link https://github.com/amplexor-drupal/xconnect We are using the Amplexor X-Connect PHP client.@endlink
 *
 * @TranslatorPlugin(
 *   id = "xconnect",
 *   label = @Translation("X-Connect translator"),
 *   description = @Translation("X-Connect Translator service."),
 *   ui = "Drupal\tmgmt_xconnect\XConnectTranslatorUi"
 * )
 */
class XConnectTranslator extends TranslatorPluginBase {

  /**
   * {@inheritdoc}
   */
  public function checkAvailable(TranslatorInterface $translator) {
    if ($translator->getSetting('order') && $translator->getSetting('connection')) {
      return AvailableResult::yes();
    }
    return AvailableResult::no(t('@translator is not available. Make sure it is properly <a href=:configured>configured</a>.', [
      '@translator' => $translator->label(),
      ':configured' => $translator->url(),
     ]));
  }

  /**
   * {@inheritdoc}
   */
  public function checkTranslatable(TranslatorInterface $translator, JobInterface $job) {
    // Anything can be exported.
    return TranslatableResult::yes();
  }

  /**
   * {@inheritdoc}
   */
  public function requestTranslation(JobInterface $job) {
    // Order name (file name) based on the Job details.
    $order_name = sprintf(
      'JobID%s_%s_%s',
      $job->id(),
      $job->getSourceLangcode(),
      $job->getTargetLangcode()
    );

    // Create a new translation request.
    $settings_order = $job->getSetting('order');
    $request = new Request(
      $job->getSourceLangcode(),
      [
        'clientId'          => $settings_order['client_id'],
        'orderNamePrefix'   => 'drupal8_order',
        'dueDate'           => $settings_order['due_date'],
        'issuedBy'          => $settings_order['issued_by'],
        'isConfidential'    => $settings_order['is_confidential'],
        'needsConfirmation' => $settings_order['needs_confirmation'],
        'needsQuotation'    => $settings_order['needs_quotation'],
      ]
    );

    // Fill in the request details:
    // The Language(s) to translate the content to:
    $request->addTargetLanguage($job->getTargetLangcode());

    // Optional instructions and reference:
    if (!empty($job->getSetting('instructions'))) {
      $request->addInstruction(Xss::filter($job->getSetting('instructions')));
    }
    if (!empty($job->getSetting('reference'))) {
      $request->setReference(Xss::filter($job->getSetting('reference')));
    }

    // Add the translation strings, add a separate file for each Job Item.
    $export = \Drupal::service('plugin.manager.tmgmt_xconnect.format')->createInstance('html');
    foreach ($job->getItems() as $item) {
      $request->addContent(
        $order_name . '_' . $item->id() . '.html',
        $export->exportJobItem($item)
      );
    }

    // Create a service object by passing the connection details.
    $settings_connection = $job->getSetting('connection');
    if ($settings_connection['port'] === 22) {
      // Transport over SSH (encryption).
      $service = new SFtpService($settings_connection);
    }
    else {
      // Transport over FTP (no encryption).
      $service = new FtpService($settings_connection);
    }

    // Send the request as a zip file.
    $result = $service->send(
      ZipFile::create(
        $request,
        tmgmt_xconnect_directory_request()
      )
    );

    // Update the Job status depending on the send request result.
    if ($result) {
      $job->submitted('Sent translation request to GCM.');
    }
    else {
      $job->addMessage(
        'There was a problem sending the translation job to %name',
        $args,
        'error'
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function hasCheckoutSettings(JobInterface $job) {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultSettings() {
    return array(
      'order' => array(
        'client_id' => '',
        'due_date' => 0,
        'issued_by' => '',
        'is_confidential' => 0,
        'needs_confirmation' => 1,
        'needs_quotation' => 0,
      ),
      'connection' => array(
        'protocol' => 'FTP',
        'port' => '21',
        'timeout' => '90',
        'hostname' => '',
        'username' => '',
        'password' => '',
        'directory_send' => 'To_LSP',
        'directory_send_processed' => 'To_LSP/processed',
        'directory_receive' => 'From_LSP',
        'directory_receive_processed' => 'From_LSP/processed',
      ),
      'cron' => array(
        'status' => 1,
        'limit' => 0,
      ),
    );
  }

}
